@@ -1,7 +1,7 @@
 package Rope;
 
 use 5.006; use strict; use warnings;
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 use Rope::Object;
 my (%META, %PRO);
 our @ISA;
@@ -19,16 +19,54 @@ BEGIN {
 				$props{properties}->{$prop} = {%{$props{properties}{$prop}}};
 				if ($props{properties}{$prop}{value} && ref $props{properties}{$prop}{value} eq 'CODE') {
 					my $cb = $props{properties}{$prop}{value};
-					$props{properties}{$prop}{value} = sub { $cb->($META{initialised}{${$self}->{identifier}}, @_) };
+					$props{properties}{$prop}{value} = sub { $cb->($META{initialised}{$caller}{${$self}->{identifier}}, @_) };
 				}
 				for (qw/trigger delete_trigger/) {
 					my $trigger = $props{properties}{$prop}{$_};
 					if (defined $trigger) {
 						$props{properties}{$prop}{$_} = sub { 
-							$trigger->($META{initialised}{${$self}->{identifier}}, @_) 
+							$trigger->($META{initialised}{$caller}{${$self}->{identifier}}, @_) 
 						};
 					}
 				}
+				for (qw/before after/) {
+					my $mod = $props{properties}{$prop}{$_};
+					if (defined $mod && scalar @{$mod}) {
+						my $cb = sub {
+							my $cb = shift;
+							$cb->($META{initialised}{$caller}{${$self}->{identifier}}, @_);
+						};
+						$props{properties}{$prop}{$_} = sub {
+							my (@params) = @_;
+							for (my $i = scalar @{$mod} - 1; $i >= 1; $i--) {
+								@params = ($cb->($mod->[$i], @params));
+							}
+							return $cb->($mod->[0], @params);
+						};
+					}
+				}
+				my $mod = $props{properties}{$prop}{around};
+				if (defined $mod && scalar @{$mod}) {
+					my $cb = sub {
+						my $cb = shift;
+						$cb->($META{initialised}{$caller}{${$self}->{identifier}}, @_);
+					};
+					$props{properties}{$prop}{around} = sub {
+						my ($orig, @params) = @_;
+						my $code = (ref($orig) || "") eq 'CODE';
+						@params = ($orig) if (!$code);
+						my @stack;
+						for (my $i = 0; $i < scalar @{$mod}; $i++) {
+							my $current = $mod->[$i];
+							my $next = $stack[-1] ? $stack[-1] : $code ? $orig : sub { $_[0] };
+							my $calling = sub {
+								return $cb->($current, $next, @_); 
+							};
+							push @stack, $calling;
+						}
+						return $stack[-1]->(@params);
+					};
+				}
 			}
 			return \%props;
 		},
@@ -96,7 +134,9 @@ BEGIN {
 					my ($prop, $options) = (shift @properties, shift @properties);
 
 					my $ref = ref $options;					
-					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } qw/initable writeable builder enumerable configurable trigger delete_trigger value/) {
+					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } 
+						qw/initable writeable builder enumerable configurable trigger delete_trigger value/
+					) {
 						$options = {
 							initable => 1,
 							enumerable => 1,
@@ -160,22 +200,25 @@ BEGIN {
 					my $merge = $PRO{clone}($META{$with});
 					$merge->{name} = $initial->{name};
 					$merge->{locked} = $initial->{locked};
-					for (keys %{$initial->{properties}}) {
-						$initial->{properties}->{$_}->{index} = ++$merge->{keys};
-						if ($merge->{properties}->{$_}) {
+					for my $prop (keys %{$initial->{properties}}) {
+						$initial->{properties}->{$prop}->{index} = ++$merge->{keys};
+						for (qw/before around after/) {
+							unshift @{$merge->{properties}->{$prop}->{$_}}, @{$initial->{properties}->{$prop}->{$_}} if $initial->{properties}->{$prop}->{$_};
+						}
+						if ($merge->{properties}->{$prop}) {
 							if ($merge->{properties}->{writeable}) {
-								$merge->{properties}->{$_} = $initial->{properties}->{$_};
+								$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 							} elsif ($merge->{properties}->{configurable}) {
-								if ((ref($merge->{properties}->{$_}->{value}) || "") eq (ref($initial->{properties}->{$_}->{value} || ""))) {
-									$merge->{properties}->{$_} = $initial->{properties}->{$_};
+								if ((ref($merge->{properties}->{$prop}->{value}) || "") eq (ref($initial->{properties}->{$prop}->{value} || ""))) {
+									$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 								} else {
-									die "Cannot include $with and change property $_ type";
+									die "Cannot include $with and change property $prop type";
 								}
 							} else {
-								die "Cannot include $with and override property $_";
+								die "Cannot include $with and override property $prop";
 							}
 						} else {
-							$merge->{properties}->{$_} = $initial->{properties}->{$_};
+							$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 						}
 					}
 					$merge->{requires} = {%{$merge->{requires}}, %{$initial->{requires}}};
@@ -197,22 +240,25 @@ BEGIN {
 					my $merge = $PRO{clone}($META{$extend});
 					$merge->{name} = $initial->{name};
 					$merge->{locked} = $initial->{locked};
-					for (keys %{$initial->{properties}}) {
-						$initial->{properties}->{$_}->{index} = ++$merge->{keys};
-						if ($merge->{properties}->{$_}) {
+					for my $prop (keys %{$initial->{properties}}) {
+						$initial->{properties}->{$prop}->{index} = ++$merge->{keys};
+						for (qw/before around after/) {
+							unshift @{$merge->{properties}->{$prop}->{$_}}, @{$initial->{properties}->{$prop}->{$_}} if $initial->{properties}->{$prop}->{$_};
+						}	
+						if ($merge->{properties}->{$prop}) {
 							if ($merge->{properties}->{writeable}) {
-								$merge->{properties}->{$_} = $initial->{properties}->{$_};
+								$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 							} elsif ($merge->{properties}->{configurable}) {
-								if ((ref($merge->{properties}->{$_}->{value}) || "") eq (ref($initial->{properties}->{$_}->{value} || ""))) {
-									$merge->{properties}->{$_} = $initial->{properties}->{$_};
+								if ((ref($merge->{properties}->{$prop}->{value}) || "") eq (ref($initial->{properties}->{$prop}->{value} || ""))) {
+									$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 								} else {
-									die "Cannot inherit $extend and change property $_ type";
+									die "Cannot inherit $extend and change property $prop type";
 								}
 							} else {
-								die "Cannot inherit $extend and override property $_";
+								die "Cannot inherit $extend and override property $prop";
 							}
 						} else {
-							$merge->{properties}->{$_} = $initial->{properties}->{$_};
+							$merge->{properties}->{$prop} = $initial->{properties}->{$prop};
 						}
 					}
 					$merge->{requires} = {%{$merge->{requires}}, %{$initial->{requires}}};
@@ -222,6 +268,27 @@ BEGIN {
 				}
 			}
 		},
+		before => sub {
+			my ($caller) = shift;
+			return sub {
+				my ($prop, $cb) = @_;
+				push @{$META{$caller}{properties}{$prop}{before}}, $cb;
+			};
+		},
+		around => sub {
+			my ($caller) = shift;
+			return sub {
+				my ($prop, $cb) = @_;
+				push @{$META{$caller}{properties}{$prop}{around}}, $cb;
+			};
+		},
+		after => sub {
+			my ($caller) = shift;
+			return sub {
+				my ($prop, $cb) = @_;
+				push @{$META{$caller}{properties}{$prop}{before}}, $cb;
+			};
+		},
 		locked => sub {
 			my ($caller) = shift;
 			return sub {
@@ -239,8 +306,16 @@ BEGIN {
 			return sub {
 				my ($self, $locked) = @_;
 				if (ref $self) {
-					delete $META{initialised}{${$self}->{identifier}};
-					$self->DESTROY;
+					return $self->DESTROY;
+				}
+			};
+		},
+		DESTROY => sub {
+			my ($caller) = shift;
+			return sub {
+				my ($self, $locked) = @_;
+				if (ref $self) {
+					delete $META{initialised}{$caller}{${$self}->{identifier}};
 					return;
 				}
 			};
@@ -251,7 +326,7 @@ BEGIN {
 				my ($class, %params) = @_;
 				my $self = \{
 					prototype => {},
-					identifier => $META{initialised}{identifier}++
+					identifier => $META{initialised}{$caller}{identifier}++
 				};
 				$self = bless $self, $caller;
 				my $build = $PRO{clone}($META{$caller});
@@ -281,7 +356,7 @@ BEGIN {
 					}
 				}
 				tie %{${$self}->{prototype}}, 'Rope::Object', $PRO{scope}($caller, $self, %{$build});
-				$META{initialised}->{${$self}->{identifier}} = $self;
+				$META{initialised}{$caller}->{${$self}->{identifier}} = $self;
 				$self->{ROPE_init}->();
 				return $self;
 			};
@@ -311,7 +386,7 @@ sub import {
 	$PRO{keyword}($caller, $_, $PRO{$_}($caller))
 		for $options->{import} 
 			? @{$options->{import}} 
-			: qw/function property properties prototyped extends with requires locked destroy new/;
+			: qw/function property properties prototyped extends with requires before around after locked destroy DESTROY new/;
 }
 
 sub new {
@@ -347,6 +422,11 @@ sub new {
 	return $PRO{new}($name)($name, %params);
 }
 
+sub get_initialised {
+	my ($self, $caller, $init) = @_;
+	return $META{initialised}{$caller}{$init};
+}
+
 1;
 
 __END__
@@ -357,7 +437,7 @@ Rope - Tied objects
 
 =head1 VERSION
 
-Version 0.14
+Version 0.15
 
 =cut
 
@@ -518,7 +598,7 @@ NOTE: traditional sub routines work and should be inherited also.
 
 =head2 extends
 
-The extends keyword allows you to extend your current definition with another object, your objext will inherit all the properties of that extended object.
+The extends keyword allows you to extend your current definition with another object, your object will inherit all the properties of that extended object.
 
 	package Ping;
 	use Rope;
@@ -526,7 +606,7 @@ The extends keyword allows you to extend your current definition with another ob
 
 =head2 with
 
-The with keyword allows you to include roles in your current object definition, your objext will inherit all the properties of that role.
+The with keyword allows you to include roles in your current object definition, your object will inherit all the properties of that role.
 
 	package Ping;
 	use Rope;
@@ -550,6 +630,82 @@ The requires keyword allows you to define properties which are required for eith
 		host => '...'
 	);
 
+=head2 before
+
+The before keyword allows you to hook in before you call a sub routine or setting a value. If a defined response is returned it will be carried through to any subsequent calls including the setting of a property.
+
+	package Locked;
+
+	use Rope;
+	use Rope::Autoload;
+
+	property count => (
+		value => 0,
+		configurable => 1,
+		enumerable => 1
+	);
+
+	function two => sub {
+		my ($self, $count) = @_;
+		$self->count = $count;
+		return $self->count;
+	};
+
+	1;
+
+	package Load;
+
+	use Rope;
+	extends 'Locked';
+
+	before count => sub {
+		my ($self, $val) = @_;
+		$val = $val * 2;
+		return $val;
+	};
+
+	before two => sub {
+		my ($self, $count) = @_;
+		... do something that doesn't return a value ...
+		return;
+	};
+
+	1;
+
+=head2 around
+
+The around keyword allows you to hook in around the calling of a sub routine or setting a value. If a defined response is returned it will be carried through to any subsequent calls including the setting of a property. To chain you must use the second param as a code reference.
+
+	package Loading;
+
+	use Rope;
+	extends 'Load';
+
+	around two => sub {
+		my ($self, $cb, $val) = @_;
+		$val = $val * 4;
+		return $cb->($val);
+	};
+
+	1;
+
+=head2 after
+
+The after keyword allows you to hook in after the calling of a sub routine or setting a value. If a defined response is returned it will be carried through to any subsequent calls including the setting of a property.
+
+	package Loaded;
+
+	use Rope;
+	extends 'Loading';
+
+	after two => sub {
+		my ($self, $val) = @_;
+		$val = $val * 4;
+		return $val;
+	};
+	
+	1;
+
 =head2 locked
 
 The locked keyword can be used to lock your Object so that it can no longer be extended with new properties/keys.
@@ -570,6 +726,7 @@ Once you have an object initialised you can toggle whether it is locked by calli
 
 	$obj->locked(0);
 
+
 =head1 METHODS
 
 =head2 new
@@ -609,6 +766,12 @@ For objects that are running in a long process, like under a web server, you wil
 
 	$knot->destroy();
 
+=head2 get_initialised
+
+This returns an existing initialised object by index set in order of initialisation, as long as it has not been destroyed by scope or calling the ->destroy explicitly, it will return undef in those cases.
+
+	Rope->get_initialised('Knot', 0);
+
 =head1 OBJECT CLASS DEFINITION
 
 	package Builder;
